@@ -4,6 +4,7 @@ use std::mem;
 use thiserror::Error;
 use tracing::debug;
 
+use super::state::get_or_insert_recv;
 use super::{Retransmits, ShouldTransmit, StreamHalf, StreamId, StreamsState, UnknownStream};
 use crate::connection::assembler::{Assembler, Chunk, IllegalOrderedRead};
 use crate::{frame, TransportError, VarInt};
@@ -18,14 +19,14 @@ pub(super) struct Recv {
 }
 
 impl Recv {
-    pub(super) fn new(initial_max_data: u64) -> Self {
-        Self {
+    pub(super) fn new(initial_max_data: u64) -> Box<Self> {
+        Box::new(Self {
             state: RecvState::default(),
             assembler: Assembler::new(),
             sent_max_stream_data: initial_max_data,
             end: 0,
             stopped: false,
-        }
+        })
     }
 
     /// Process a STREAM frame
@@ -215,15 +216,16 @@ impl<'a> Chunks<'a> {
         streams: &'a mut StreamsState,
         pending: &'a mut Retransmits,
     ) -> Result<Self, ReadableError> {
-        let entry = match streams.recv.entry(id) {
+        let mut entry = match streams.recv.entry(id) {
             Entry::Occupied(entry) => entry,
             Entry::Vacant(_) => return Err(ReadableError::UnknownStream),
         };
 
-        let mut recv = match entry.get().stopped {
-            true => return Err(ReadableError::UnknownStream),
-            false => entry.remove(),
-        };
+        let mut recv =
+            match get_or_insert_recv(streams.stream_receive_window)(entry.get_mut()).stopped {
+                true => return Err(ReadableError::UnknownStream),
+                false => entry.remove().unwrap(), // this can't fail due to the previous get_or_insert_with
+            };
 
         recv.assembler.ensure_ordering(ordered)?;
         Ok(Self {
@@ -313,7 +315,7 @@ impl<'a> Chunks<'a> {
                 self.pending.max_stream_data.insert(self.id);
             }
             // Return the stream to storage for future use
-            self.streams.recv.insert(self.id, rs);
+            self.streams.recv.insert(self.id, Some(rs));
         }
 
         // Issue connection-level flow control credit for any data we read regardless of state
@@ -331,7 +333,7 @@ impl<'a> Drop for Chunks<'a> {
 }
 
 enum ChunksState {
-    Readable(Recv),
+    Readable(Box<Recv>),
     Reset(VarInt),
     Finished,
     Finalized,
