@@ -28,8 +28,8 @@ use crate::{
         EndpointEventInner, IssuedCid,
     },
     transport_parameters::TransportParameters,
-    ResetToken, RetryToken, Side, Transmit, TransportConfig, TransportError,
-    INITIAL_MAX_UDP_PAYLOAD_SIZE, MAX_CID_SIZE, MIN_INITIAL_SIZE, RESET_TOKEN_SIZE,
+    ResetToken, RetryToken, Side, Transmit, TransportConfig, TransportError, INITIAL_MTU,
+    MAX_CID_SIZE, MIN_INITIAL_SIZE, RESET_TOKEN_SIZE,
 };
 
 /// The main entry point to the library
@@ -61,13 +61,33 @@ pub struct Endpoint {
     local_cid_generator: Box<dyn ConnectionIdGenerator>,
     config: Arc<EndpointConfig>,
     server_config: Option<Arc<ServerConfig>>,
+    /// Whether the underlying UDP socket promises not to fragment packets
+    allow_mtud: bool,
+    /// The contents length for packets in the transmits queue
+    transmit_queue_contents_len: usize,
+    /// The socket buffer aggregated contents length
+    /// `transmit_queue_contents_len` + `socket_buffer_fill` represents the total contents length
+    /// of outstanding outgoing packets.
+    socket_buffer_fill: usize,
 }
 
+/// The maximum size of content length of packets in the outgoing transmit queue. Transmit packets
+/// generated from the endpoint (retry, initial close, stateless reset and version negotiation)
+/// can be dropped when this limit is being execeeded.
+/// Chose to represent 100 MB of data.
+const MAX_TRANSMIT_QUEUE_CONTENTS_LEN: usize = 100_000_000;
+
 impl Endpoint {
     /// Create a new endpoint
     ///
-    /// Returns `Err` if the configuration is invalid.
-    pub fn new(config: Arc<EndpointConfig>, server_config: Option<Arc<ServerConfig>>) -> Self {
+    /// `allow_mtud` enables path MTU detection when requested by `Connection` configuration for
+    /// better performance. This requires that outgoing packets are never fragmented, which can be
+    /// achieved via e.g. the `IPV6_DONTFRAG` socket option.
+    pub fn new(
+        config: Arc<EndpointConfig>,
+        server_config: Option<Arc<ServerConfig>>,
+        allow_mtud: bool,
+    ) -> Self {
         Self {
             rng: StdRng::from_entropy(),
             transmits: VecDeque::new(),
@@ -79,13 +99,18 @@ impl Endpoint {
             local_cid_generator: (config.connection_id_generator_factory.as_ref())(),
             config,
             server_config,
+            allow_mtud,
+            transmit_queue_contents_len: 0,
+            socket_buffer_fill: 0,
         }
     }
 
     /// Get the next packet to transmit
     #[must_use]
     pub fn poll_transmit(&mut self) -> Option<Transmit> {
-        self.transmits.pop_front()
+        let t = self.transmits.pop_front();
+        self.decrement_transmit_queue_contents_len(t.as_ref().map_or(0, |t| t.contents.len()));
+        t
     }
 
     /// Replace the server configuration, affecting new incoming connections only
@@ -166,9 +191,12 @@ impl Endpoint {
                     debug!("dropping packet with unsupported version");
                     return None;
                 }
+                if self.stateless_packets_supressed() {
+                    return None;
+                }
                 trace!("sending version negotiation");
                 // Negotiate versions
-                let mut buf = Vec::<u8>::new();
+                let mut buf = BytesMut::new();
                 Header::VersionNegotiate {
                     random: self.rng.gen::<u8>() | 0x40,
                     src_cid: dst_cid,
@@ -184,10 +212,11 @@ impl Endpoint {
                 for &version in &self.config.supported_versions {
                     buf.write(version);
                 }
+                self.increment_transmit_queue_contents_len(buf.len());
                 self.transmits.push_back(Transmit {
                     destination: remote,
                     ecn: None,
-                    contents: buf,
+                    contents: buf.freeze(),
                     segment_size: None,
                     src_ip: local_ip,
                 });
@@ -314,6 +343,9 @@ impl Endpoint {
         addresses: FourTuple,
         dst_cid: &ConnectionId,
     ) {
+        if self.stateless_packets_supressed() {
+            return;
+        }
         /// Minimum amount of padding for the stateless reset to look like a short-header packet
         const MIN_PADDING_LEN: usize = 5;
 
@@ -331,7 +363,7 @@ impl Endpoint {
             "sending stateless reset for {} to {}",
             dst_cid, addresses.remote
         );
-        let mut buf = Vec::<u8>::new();
+        let mut buf = BytesMut::new();
         // Resets with at least this much padding can't possibly be distinguished from real packets
         const IDEAL_MIN_PADDING_LEN: usize = MIN_PADDING_LEN + MAX_CID_SIZE;
         let padding_len = if max_padding_len <= IDEAL_MIN_PADDING_LEN {
@@ -339,18 +371,18 @@ impl Endpoint {
         } else {
             self.rng.gen_range(IDEAL_MIN_PADDING_LEN..max_padding_len)
         };
-        buf.reserve_exact(padding_len + RESET_TOKEN_SIZE);
+        buf.reserve(padding_len + RESET_TOKEN_SIZE);
         buf.resize(padding_len, 0);
         self.rng.fill_bytes(&mut buf[0..padding_len]);
         buf[0] = 0b0100_0000 | buf[0] >> 2;
         buf.extend_from_slice(&ResetToken::new(&*self.config.reset_key, dst_cid));
 
         debug_assert!(buf.len() < inciting_dgram_len);
-
+        self.increment_transmit_queue_contents_len(buf.len());
         self.transmits.push_back(Transmit {
             destination: addresses.remote,
             ecn: None,
-            contents: buf,
+            contents: buf.freeze(),
             segment_size: None,
             src_ip: addresses.local_ip,
         });
@@ -366,9 +398,12 @@ impl Endpoint {
         if self.is_full() {
             return Err(ConnectError::TooManyConnections);
         }
-        if remote.port() == 0 {
+        if remote.port() == 0 || remote.ip().is_unspecified() {
             return Err(ConnectError::InvalidRemoteAddress(remote));
         }
+        if !self.config.supported_versions.contains(&config.version) {
+            return Err(ConnectError::UnsupportedVersion);
+        }
 
         let remote_id = RandomConnectionIdGenerator::new(MAX_CID_SIZE).generate_cid();
         trace!(initial_dcid = %remote_id);
@@ -435,6 +470,35 @@ impl Endpoint {
         }
     }
 
+    /// Limiting the memory usage for items queued in the outgoing queue from endpoint
+    /// generated packets. Otherwise, we may see a build-up of the queue under test with
+    /// flood of initial packets against the endpoint. The sender with the sender-limiter
+    /// may not keep up the pace of these packets queued into the queue.
+    fn stateless_packets_supressed(&self) -> bool {
+        self.transmit_queue_contents_len
+            .saturating_add(self.socket_buffer_fill)
+            >= MAX_TRANSMIT_QUEUE_CONTENTS_LEN
+    }
+
+    /// Increment the contents length in the transmit queue.
+    fn increment_transmit_queue_contents_len(&mut self, contents_len: usize) {
+        self.transmit_queue_contents_len = self
+            .transmit_queue_contents_len
+            .saturating_add(contents_len);
+    }
+
+    /// Decrement the contents length in the transmit queue.
+    fn decrement_transmit_queue_contents_len(&mut self, contents_len: usize) {
+        self.transmit_queue_contents_len = self
+            .transmit_queue_contents_len
+            .saturating_sub(contents_len);
+    }
+
+    /// Set the `socket_buffer_fill` to the input `len`
+    pub fn set_socket_buffer_fill(&mut self, len: usize) {
+        self.socket_buffer_fill = len;
+    }
+
     fn handle_first_packet(
         &mut self,
         now: Instant,
@@ -509,6 +573,9 @@ impl Endpoint {
 
         let (retry_src_cid, orig_dst_cid) = if server_config.use_retry {
             if token.is_empty() {
+                if self.stateless_packets_supressed() {
+                    return None;
+                }
                 // First Initial
                 let mut random_bytes = vec![0u8; RetryToken::RANDOM_BYTES_LEN];
                 self.rng.fill_bytes(&mut random_bytes);
@@ -526,16 +593,17 @@ impl Endpoint {
                     version,
                 };
 
-                let mut buf = Vec::new();
+                let mut buf = BytesMut::new();
                 let encode = header.encode(&mut buf);
                 buf.put_slice(&token);
                 buf.extend_from_slice(&server_config.crypto.retry_tag(version, &dst_cid, &buf));
                 encode.finish(&mut buf, &*crypto.header.local, None);
 
+                self.increment_transmit_queue_contents_len(buf.len());
                 self.transmits.push_back(Transmit {
                     destination: addresses.remote,
                     ecn: None,
-                    contents: buf,
+                    contents: buf.freeze(),
                     segment_size: None,
                     src_ip: addresses.local_ip,
                 });
@@ -598,14 +666,7 @@ impl Endpoint {
         if dst_cid.len() != 0 {
             self.connection_ids_initial.insert(dst_cid, ch);
         }
-        match conn.handle_first_packet(
-            now,
-            addresses.remote,
-            ecn,
-            packet_number as u64,
-            packet,
-            rest,
-        ) {
+        match conn.handle_first_packet(now, addresses.remote, ecn, packet_number, packet, rest) {
             Ok(()) => {
                 trace!(id = ch.0, icid = %dst_cid, "connection incoming");
                 Some((ch, conn))
@@ -646,6 +707,7 @@ impl Endpoint {
             self.local_cid_generator.as_ref(),
             now,
             version,
+            self.allow_mtud,
         );
 
         let id = self.connections.insert(ConnectionMeta {
@@ -674,6 +736,9 @@ impl Endpoint {
         local_id: &ConnectionId,
         reason: TransportError,
     ) {
+        if self.stateless_packets_supressed() {
+            return;
+        }
         let number = PacketNumber::U8(0);
         let header = Header::Initial {
             dst_cid: *remote_id,
@@ -683,11 +748,10 @@ impl Endpoint {
             version,
         };
 
-        let mut buf = Vec::<u8>::new();
+        let mut buf = BytesMut::new();
         let partial_encode = header.encode(&mut buf);
-        let max_len = INITIAL_MAX_UDP_PAYLOAD_SIZE as usize
-            - partial_encode.header_len
-            - crypto.packet.local.tag_len();
+        let max_len =
+            INITIAL_MTU as usize - partial_encode.header_len - crypto.packet.local.tag_len();
         frame::Close::from(reason).encode(&mut buf, max_len);
         buf.resize(buf.len() + crypto.packet.local.tag_len(), 0);
         partial_encode.finish(
@@ -695,16 +759,22 @@ impl Endpoint {
             &*crypto.header.local,
             Some((0, &*crypto.packet.local)),
         );
+        self.increment_transmit_queue_contents_len(buf.len());
         self.transmits.push_back(Transmit {
             destination: addresses.remote,
             ecn: None,
-            contents: buf,
+            contents: buf.freeze(),
             segment_size: None,
             src_ip: addresses.local_ip,
         })
     }
 
-    /// Unconditionally reject future incoming connections
+    /// Reject new incoming connections without affecting existing connections
+    ///
+    /// Convenience short-hand for using
+    /// [`set_server_config`](Self::set_server_config) to update
+    /// [`concurrent_connections`](ServerConfig::concurrent_connections) to
+    /// zero.
     pub fn reject_new_connections(&mut self) {
         if let Some(config) = self.server_config.as_mut() {
             Arc::make_mut(config).concurrent_connections(0);
@@ -782,7 +852,7 @@ pub(crate) struct ConnectionMeta {
 pub struct ConnectionHandle(pub usize);
 
 impl From<ConnectionHandle> for usize {
-    fn from(x: ConnectionHandle) -> usize {
+    fn from(x: ConnectionHandle) -> Self {
         x.0
     }
 }
@@ -837,7 +907,7 @@ pub enum ConnectError {
     /// Use `Endpoint::connect_with` to specify a client configuration.
     #[error("no default client config")]
     NoDefaultClientConfig,
-    /// The cryptographic layer does not support the specified QUIC version
+    /// The local endpoint does not support the QUIC version specified in the client configuration
     #[error("unsupported QUIC version")]
     UnsupportedVersion,
 }
