@@ -18,14 +18,16 @@ use crate::{
 // across QUIC versions), which gives us the destination CID and allows us
 // to inspect the version and packet type (which depends on the version).
 // This information allows us to fully decode and decrypt the packet.
+#[allow(unreachable_pub)] // fuzzing only
 #[derive(Debug)]
 pub struct PartialDecode {
     plain_header: PlainHeader,
     buf: io::Cursor<BytesMut>,
 }
 
+#[allow(clippy::len_without_is_empty)]
 impl PartialDecode {
-    #![allow(clippy::len_without_is_empty)]
+    #[allow(unreachable_pub)] // fuzzing only
     pub fn new(
         bytes: BytesMut,
         local_cid_len: usize,
@@ -101,6 +103,7 @@ impl PartialDecode {
     }
 
     /// Length of QUIC packet being decoded
+    #[allow(unreachable_pub)] // fuzzing only
     pub fn len(&self) -> usize {
         self.buf.get_ref().len()
     }
@@ -222,7 +225,7 @@ pub(crate) struct Packet {
 }
 
 impl Packet {
-    pub fn reserved_bits_valid(&self) -> bool {
+    pub(crate) fn reserved_bits_valid(&self) -> bool {
         let mask = match self.header {
             Header::Short { .. } => SHORT_RESERVED_BITS,
             _ => LONG_RESERVED_BITS,
@@ -266,7 +269,7 @@ pub(crate) enum Header {
 }
 
 impl Header {
-    pub(crate) fn encode(&self, w: &mut Vec<u8>) -> PartialEncode {
+    pub(crate) fn encode(&self, w: &mut BytesMut) -> PartialEncode {
         use self::Header::*;
         let start = w.len();
         match *self {
@@ -365,10 +368,7 @@ impl Header {
 
     /// Whether the packet is encrypted on the wire
     pub(crate) fn is_protected(&self) -> bool {
-        !matches!(
-            *self,
-            Header::Retry { .. } | Header::VersionNegotiate { .. }
-        )
+        !matches!(*self, Self::Retry { .. } | Self::VersionNegotiate { .. })
     }
 
     pub(crate) fn number(&self) -> Option<PacketNumber> {
@@ -401,13 +401,13 @@ impl Header {
 
     pub(crate) fn key_phase(&self) -> bool {
         match *self {
-            Header::Short { key_phase, .. } => key_phase,
+            Self::Short { key_phase, .. } => key_phase,
             _ => false,
         }
     }
 
     pub(crate) fn is_short(&self) -> bool {
-        matches!(*self, Header::Short { .. })
+        matches!(*self, Self::Short { .. })
     }
 
     pub(crate) fn is_1rtt(&self) -> bool {
@@ -417,7 +417,7 @@ impl Header {
     pub(crate) fn is_0rtt(&self) -> bool {
         matches!(
             *self,
-            Header::Long {
+            Self::Long {
                 ty: LongType::ZeroRtt,
                 ..
             }
@@ -437,8 +437,8 @@ impl Header {
 }
 
 pub(crate) struct PartialEncode {
-    pub start: usize,
-    pub header_len: usize,
+    pub(crate) start: usize,
+    pub(crate) header_len: usize,
     // Packet number length, payload length needed
     pn: Option<(usize, bool)>,
 }
@@ -450,7 +450,7 @@ impl PartialEncode {
         header_crypto: &dyn crypto::HeaderKey,
         crypto: Option<(u64, &dyn crypto::PacketKey)>,
     ) {
-        let PartialEncode { header_len, pn, .. } = self;
+        let Self { header_len, pn, .. } = self;
         let (pn_len, write_len) = match pn {
             Some((pn_len, write_len)) => (pn_len, write_len),
             None => return,
@@ -545,7 +545,7 @@ impl PlainHeader {
                 return Err(PacketDecodeError::InvalidHeader("cid out of bounds"));
             }
 
-            Ok(PlainHeader::Short {
+            Ok(Self::Short {
                 spin,
                 dst_cid: ConnectionId::from_buf(buf, local_cid_len),
             })
@@ -560,7 +560,7 @@ impl PlainHeader {
             // TODO: Support long CIDs for compatibility with future QUIC versions
             if version == 0 {
                 let random = first & !LONG_HEADER_FORM;
-                return Ok(PlainHeader::VersionNegotiate {
+                return Ok(Self::VersionNegotiate {
                     random,
                     dst_cid,
                     src_cid,
@@ -585,7 +585,7 @@ impl PlainHeader {
                     buf.advance(token_len);
 
                     let len = buf.get_var()?;
-                    Ok(PlainHeader::Initial {
+                    Ok(Self::Initial {
                         dst_cid,
                         src_cid,
                         token_pos: token_start..token_start + token_len,
@@ -593,12 +593,12 @@ impl PlainHeader {
                         version,
                     })
                 }
-                LongHeaderType::Retry => Ok(PlainHeader::Retry {
+                LongHeaderType::Retry => Ok(Self::Retry {
                     dst_cid,
                     src_cid,
                     version,
                 }),
-                LongHeaderType::Standard(ty) => Ok(PlainHeader::Long {
+                LongHeaderType::Standard(ty) => Ok(Self::Long {
                     ty,
                     dst_cid,
                     src_cid,
@@ -623,13 +623,13 @@ impl PacketNumber {
     pub(crate) fn new(n: u64, largest_acked: u64) -> Self {
         let range = (n - largest_acked) * 2;
         if range < 1 << 8 {
-            PacketNumber::U8(n as u8)
+            Self::U8(n as u8)
         } else if range < 1 << 16 {
-            PacketNumber::U16(n as u16)
+            Self::U16(n as u16)
         } else if range < 1 << 24 {
-            PacketNumber::U24(n as u32)
+            Self::U24(n as u32)
         } else if range < 1 << 32 {
-            PacketNumber::U32(n as u32)
+            Self::U32(n as u32)
         } else {
             panic!("packet number too large to encode")
         }
@@ -655,7 +655,7 @@ impl PacketNumber {
         }
     }
 
-    pub(crate) fn decode<R: Buf>(len: usize, r: &mut R) -> Result<PacketNumber, PacketDecodeError> {
+    pub(crate) fn decode<R: Buf>(len: usize, r: &mut R) -> Result<Self, PacketDecodeError> {
         use self::PacketNumber::*;
         let pn = match len {
             1 => U8(r.get()?),
@@ -736,7 +736,7 @@ impl LongHeaderType {
 }
 
 impl From<LongHeaderType> for u8 {
-    fn from(ty: LongHeaderType) -> u8 {
+    fn from(ty: LongHeaderType) -> Self {
         use self::{LongHeaderType::*, LongType::*};
         match ty {
             Initial => LONG_HEADER_FORM | FIXED_BIT,
@@ -754,6 +754,7 @@ pub(crate) enum LongType {
     ZeroRtt,
 }
 
+#[allow(unreachable_pub)] // fuzzing only
 #[derive(Debug, Error, Clone, Eq, PartialEq, Ord, PartialOrd, Hash)]
 pub enum PacketDecodeError {
     #[error("unsupported version {version:x}")]
@@ -768,7 +769,7 @@ pub enum PacketDecodeError {
 
 impl From<coding::UnexpectedEnd> for PacketDecodeError {
     fn from(_: coding::UnexpectedEnd) -> Self {
-        PacketDecodeError::InvalidHeader("unexpected end of packet")
+        Self::InvalidHeader("unexpected end of packet")
     }
 }
 
@@ -791,9 +792,7 @@ pub enum SpaceId {
 
 impl SpaceId {
     pub fn iter() -> impl Iterator<Item = Self> {
-        [SpaceId::Initial, SpaceId::Handshake, SpaceId::Data]
-            .iter()
-            .cloned()
+        [Self::Initial, Self::Handshake, Self::Data].iter().cloned()
     }
 }
 
@@ -845,7 +844,7 @@ mod tests {
 
         let dcid = ConnectionId::new(&hex!("06b858ec6f80452b"));
         let client = initial_keys(Version::V1, &dcid, Side::Client);
-        let mut buf = Vec::new();
+        let mut buf = BytesMut::new();
         let header = Header::Initial {
             number: PacketNumber::U8(0),
             src_cid: ConnectionId::new(&[]),
@@ -863,7 +862,7 @@ mod tests {
         );
 
         for byte in &buf {
-            print!("{:02x}", byte);
+            print!("{byte:02x}");
         }
         println!();
         assert_eq!(
@@ -876,7 +875,7 @@ mod tests {
 
         let server = initial_keys(Version::V1, &dcid, Side::Server);
         let supported_versions = DEFAULT_SUPPORTED_VERSIONS.to_vec();
-        let decode = PartialDecode::new(buf.as_slice().into(), 0, &supported_versions, false)
+        let decode = PartialDecode::new(buf, 0, &supported_versions, false)
             .unwrap()
             .0;
         let mut packet = decode.finish(Some(&*server.header.remote)).unwrap();
